#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <ctime>
#include <cmath>
#include <cstring>

double boxv[8][3] = {
    {1,1,1},    // 0 
    {1,1,-1},   // 1
    {1,-1,1},   // 2
    {1,-1,-1},  // 3
    {-1,1,1},   // 4
    {-1,1,-1},  // 5
    {-1,-1,1},  // 6
    {-1,-1,-1}  // 7
};

int boxf[6][4] = {
    {0,4,6,2},
    {1,3,7,5},
    {0,2,3,1},
    {0,1,5,4},
    {4,5,7,6},
    {2,6,7,3}
};

void drawBox(double r,double g,double b){
    glBegin( GL_QUADS );
    for(int i=0;i<6;i++){       // for every face
        for(int j=0;j<4;j++){   // for every point of that face
            glColor3d( 
                boxv[ boxf[i][j] ][0] > 0 ? 0 : 1,
                boxv[ boxf[i][j] ][1] > 0 ? 0 : 1,
                boxv[ boxf[i][j] ][2] > 0 ? 0 : 1
            );
            glVertex3d(
                boxv[ boxf[i][j] ][0],
                boxv[ boxf[i][j] ][1],
                boxv[ boxf[i][j] ][2]  
            );
        }
    }
    glEnd( );
}


// main to get the whole thing going.
int main( int argc, char** argv ) {

    double pi = atan(1)*4.0;

    printf("pi is: %f\n",pi);

    srand(time(NULL));

    // First, initialize SDL's video subsystem. 
    if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) {
        printf("failed to init video: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    // Information about the current video settings. 
    const SDL_VideoInfo* info = SDL_GetVideoInfo( );
    if(!info){
        printf("failed to query video: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    // setup opengl params
    //SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    //SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    //SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
    
    int flags = SDL_OPENGL; // | SDL_FULLSCREEN;
    int width = 640;
    int height = 480;
    int bpp = info->vfmt->BitsPerPixel;

    // Set the video mode
    if( SDL_SetVideoMode( width, height, bpp, flags ) == 0 ) {
        printf("failed init video mode: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    float ratio = (float) width / (float) height;

    glShadeModel( GL_SMOOTH );
    glCullFace( GL_BACK );  // remove triangles not facing us.
    //glFrontFace( GL_CCW );  // how we define 'front"
    glEnable( GL_CULL_FACE );
    //glEnable(GL_LIGHTING);  // enable lighting.
    glClearColor( 0, 0, 0, 0 ); // black
    glViewport( 0, 0, width, height );  // our "screen"
    glMatrixMode( GL_PROJECTION );  // set projection 
    glLoadIdentity( );      // load identity matrix into projection matrix.
        // fov, aspect ratio, near z, far z.
    gluPerspective( 90.0, ratio, 1, 1024.0*10 ); // setup perspective matrix.
    
    double angle = 0;

    for(;;){

        // read key strokes
        SDL_Event event;
        while( SDL_PollEvent( &event ) ) {
            switch( event.type ) {
            case SDL_KEYDOWN:
                printf("The %s(%d) key was pressed!\n",
                    SDL_GetKeyName(event.key.keysym.sym),event.key.keysym.sym);
                if(event.key.keysym.sym == SDLK_ESCAPE){
                    SDL_Quit( );
                    return 0;
                }
            break;
            case SDL_KEYUP:
                printf("The %s(%d) key was released!\n",
                    SDL_GetKeyName(event.key.keysym.sym),event.key.keysym.sym);
                break;            
           case SDL_MOUSEMOTION:
                printf("Mouse moved by %d,%d to (%d,%d)\n", 
                    event.motion.xrel, event.motion.yrel,
                    event.motion.x, event.motion.y);
                break;
            case SDL_MOUSEBUTTONDOWN:
                printf("Mouse button %d pressed at (%d,%d)\n",
                    event.button.button, event.button.x, event.button.y);
                break;
            case SDL_QUIT:      // window closed.
                SDL_Quit( );
                return 0; 
                break;
            }
        }

        

        // draw stuff to screen...
        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
        glMatrixMode( GL_MODELVIEW );
        glLoadIdentity( );

        angle++;
        glTranslated( 0.0, 0.0, -4.0 );
        glRotated(angle,0,.7,.7);

        // key thing of our code.
        //glBegin( GL_TRIANGLES );
        //    glColor3f( 255, 0, 0 );
        //    glVertex2f( -0.5,-0.5 );
        //    glColor3f( 0, 255, 0 );
        //    glVertex2f( 0.5,-0.5 );
        //    glColor3f( 0, 0, 255 );
        //    glVertex2f( 0.5,0.5 );
        //glEnd( );

        drawBox(0.3,0.7,0.8);        

        SDL_GL_SwapBuffers( );
        SDL_Delay(30);
    }
    
    SDL_Quit( );
    return 0;
}



