#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <ctime>
#include <cmath>
#include <cstring>

double boxv[8][3] = {
    {1,1,1},    // 0 
    {1,1,-1},   // 1
    {1,-1,1},   // 2
    {1,-1,-1},  // 3
    {-1,1,1},   // 4
    {-1,1,-1},  // 5
    {-1,-1,1},  // 6
    {-1,-1,-1}  // 7
};

int boxf[6][4] = {
    {0,4,6,2},
    {1,3,7,5},
    {0,2,3,1},
    {0,1,5,4},
    {4,5,7,6},
    {2,6,7,3}
};

void drawBox(){
    glBegin( GL_QUADS );
    for(int i=0;i<6;i++){       // for every face
        for(int j=0;j<4;j++){   // for every point of that face
            glColor3d( 
                boxv[ boxf[i][j] ][0] > 0 ? 0 : 1,
                boxv[ boxf[i][j] ][1] > 0 ? 0 : 1,
                boxv[ boxf[i][j] ][2] > 0 ? 0 : 1
            );
            glVertex3d(
                boxv[ boxf[i][j] ][0],
                boxv[ boxf[i][j] ][1],
                boxv[ boxf[i][j] ][2]  
            );
        }
    }
    glEnd( );
}


// main to get the whole thing going.
int main( int argc, char** argv ) {

    double pi = atan(1)*4.0;

    printf("pi is: %f\n",pi);

    srand(time(NULL));

    // First, initialize SDL's video subsystem. 
    if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) {
        printf("failed to init video: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    // Information about the current video settings. 
    const SDL_VideoInfo* info = SDL_GetVideoInfo( );
    if(!info){
        printf("failed to query video: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    // setup opengl params
    //SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
    //SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
    //SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
    
    int flags = SDL_OPENGL; // | SDL_FULLSCREEN;
    int width = 640;
    int height = 480;
    int bpp = info->vfmt->BitsPerPixel;

    // Set the video mode
    if( SDL_SetVideoMode( width, height, bpp, flags ) == 0 ) {
        printf("failed init video mode: %s",SDL_GetError());
        SDL_Quit( );
        return -1;
    }

    float ratio = (float) width / (float) height;

    glShadeModel( GL_SMOOTH );
    //glShadeModel(GL_FLAT);
    glCullFace( GL_BACK );  // remove triangles not facing us.
    //glFrontFace( GL_CCW );  // how we define 'front"
    glEnable( GL_CULL_FACE );
    //glEnable(GL_LIGHTING);  // enable lighting.
    glEnable(GL_DEPTH_TEST);        // do z-buffering.
    glClearColor( 0, 0, 0, 0 ); // black
    glViewport( 0, 0, width, height );  // our "screen"
    glMatrixMode( GL_PROJECTION );  // set projection 
    glLoadIdentity( );      // load identity matrix into projection matrix.
        // fov, aspect ratio, near z, far z.
    gluPerspective( 90.0, ratio, 1, 1024.0*10 ); // setup perspective matrix.
    

    double eye_yaw,eye_roll,eye_pitch,eye_x,eye_y,eye_z;
    eye_yaw = eye_roll = eye_pitch = eye_x = eye_y = eye_z = 0;
    
    int allkeys[0x1000];
    for(int i=0;i<0x1000;i++)
        allkeys[i] = 0;

    for(;;){

        // read key strokes
        SDL_Event event;
        while( SDL_PollEvent( &event ) ) {
            switch( event.type ) {
            case SDL_KEYDOWN:
                printf("The %s(%d) key was pressed!\n",
                    SDL_GetKeyName(event.key.keysym.sym),event.key.keysym.sym);
                allkeys[event.key.keysym.sym] = 1;
                if(event.key.keysym.sym == SDLK_ESCAPE){
                    SDL_Quit( );
                    return 0;
                }
            break;
            case SDL_KEYUP:
                allkeys[event.key.keysym.sym] = 0;
                printf("The %s(%d) key was released!\n",
                    SDL_GetKeyName(event.key.keysym.sym),event.key.keysym.sym);
                break;            
           case SDL_MOUSEMOTION:
                printf("Mouse moved by %d,%d to (%d,%d)\n", 
                    event.motion.xrel, event.motion.yrel,
                    event.motion.x, event.motion.y);

                eye_yaw += -event.motion.xrel;
                eye_pitch += event.motion.yrel;
                if(eye_pitch < -35)     eye_pitch=-35;
                if(eye_pitch > 35)      eye_pitch=35;

                break;
            case SDL_MOUSEBUTTONDOWN:
                printf("Mouse button %d pressed at (%d,%d)\n",
                    event.button.button, event.button.x, event.button.y);
                break;
            case SDL_QUIT:      // window closed.
                SDL_Quit( );
                return 0; 
                break;
            }
        }

//The left(276) key was released!
//The right(275) key was pressed!

        // move our "player"
        if( allkeys[273] ){     // up
            double s = sin(eye_yaw * pi / 180);
            double c = cos(eye_yaw * pi / 180);
            eye_x = eye_x - c * 5.0;
            eye_z = eye_z - s * 5.0;
        }
        // move our "player"
        if( allkeys[274] ){     // down
            double s = sin(eye_yaw * pi / 180);
            double c = cos(eye_yaw * pi / 180);
            eye_x = eye_x + c * 5.0;
            eye_z = eye_z + s * 5.0;
        }
        if(allkeys[276]){   // left
            eye_yaw -= 2;
        }
        if(allkeys[275]){   // right
            eye_yaw += 2;
        }

        

        // draw stuff to screen...
        glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
        glMatrixMode( GL_MODELVIEW );
        glLoadIdentity( );

        glPushMatrix();

        // eye

        glRotated(eye_pitch,1,0,0);
        glRotated(eye_yaw,0,1,0);
    
        glRotated(90,0,1,0);        
        glTranslated(eye_x,eye_y,eye_z);

        glPushMatrix();
            glTranslated(0,-200,0);
            glScaled(3000,3000,3000);
            glScaled(1,0.02,1);
            drawBox();
        glPopMatrix();


        for(double theta = 0;theta < pi*2; theta += pi / 8){
            glPushMatrix();
            glRotated(theta * 180/pi,0,1,0);
            glTranslated( 0.0, 0.0, -800.0 );    
            glScaled(100,100,100);
            glScaled(0.4,1.4,0.2);
            drawBox();
            glPopMatrix();
        }
        
        for(double theta = pi/16;theta < pi*2; theta += pi / 8){
            glPushMatrix();
            glRotated(theta * 180/pi,0,1,0);
            glTranslated( 0.0, 140, -800.0 );
            glRotated(90,1,0,0);
            glRotated(90,0,0,1);
            glScaled(100,100,100);
            glScaled(0.4,1.4,0.2);
            drawBox();
            glPopMatrix();            
        }


        for(int layer = 0; layer < 7; layer++){
            for(double theta = layer%2 ? pi/128 : 0 ;theta < pi*2; theta += pi / 64){
                glPushMatrix();
                glRotated(theta * 180/pi,0,1,0);
                glTranslated( 0.0, layer*14 -100 , -1200.0 );                
                glRotated(90,1,0,0);
                glRotated(90,0,0,1);
                glScaled(14,14,14);
                glScaled(0.6,1.4,0.4);
                drawBox();
                glPopMatrix();
            }
        }


        glPopMatrix();

        SDL_GL_SwapBuffers( );
        SDL_Delay(30);
    }
    
    SDL_Quit( );
    return 0;
}



